package model;

import java.util.ArrayList;

public class Rook extends Piece {

    public Rook(Position p, int color) {
        super(p, color);
    }

    public ArrayList<Position> getAvailableMoves(Board b) {

        // creates a dynamic list of available moves
        ArrayList<Position> moves = new ArrayList<>();
        Position arrival;

        // list of the 4 rook directions
        int[] all_x = {0, 1, 0, -1};
        int[] all_y = {1, 0, -1, 0};

        for(int n = 0 ; n < all_x.length ; n++) { // for each direction
            int x = all_x[n];
            int y = all_y[n];
            boolean blocked = false;
            int i = 1;
            while (i < 8 && !blocked) { // continues "as a ray" in that direction until blocked
                arrival = new Position(p.getX() + (x * i), p.getY() + (y * i));
                if (b.isEnemy(arrival, p) || b.isFree(arrival)) {
                    moves.add(arrival);
                }
                if(b.isKing(arrival) && !b.getPiece(arrival).hasBeenMoved() && !hasBeenMoved) { // Castling
                    moves.add(arrival);
                }
                if (!b.isFree(arrival)) {
                    blocked = true;
                }
                i++;
            }
        }

        // Full explicit directions version : ----------------------------
        /*
        int i = 1;
        boolean left = false;
        boolean right = false;
        boolean top = false;
        boolean down = false;

        while (i < 8) {

            //left
            arrival = new Position(p.getX() + i, p.getY());
            if (Position.isCorrect(arrival)) {
                if (!left) {
                    if (b.isEnemy(arrival, p) || b.isFree(arrival)) {
                        moves.add(arrival);
                    }
                    if (b.isKing(arrival)) {
                        moves.add(arrival);
                    }
                }
                if (!b.isFree(arrival)) {
                    left = true;
                }
            }

            //right
            arrival = new Position(p.getX() - i, p.getY());
            if (Position.isCorrect(arrival)) {
                if (!right) {
                    if (b.isEnemy(arrival, p) || b.isFree(arrival)) {
                        moves.add(arrival);
                    }
                    if (b.isKing(arrival)) {
                        moves.add(arrival);
                    }
                }
                if (!b.isFree(arrival)) {
                    right = true;
                }
            }

            arrival = new Position(p.getX(), p.getY() + i);
            if (Position.isCorrect(arrival)) {
                if (!top) {
                    if (b.isEnemy(arrival, p)|| b.isFree(arrival)) {
                        moves.add(arrival);
                    }
                }
                if (!b.isFree(arrival)) {
                    top = true;
                }
            }

            arrival = new Position(p.getX(), p.getY() - i);
            if(Position.isCorrect(arrival)||b.isFree(arrival)) {
                if (!down) {
                    if (b.isEnemy(arrival, p)) {
                        moves.add(arrival);
                    }
                }
                if (!b.isFree(arrival)) {
                    down = true;
                }
            }
            i++;
        }
        */
        // ------------------------------------------------------------

        return moves;
    }

    public Piece newInstanceCopy() {
        Rook rook = new Rook(p, color);
        rook.setHasBeenMoved(hasBeenMoved);
        return rook;
    }
}
